/****************************************************************************
  Copyright (c) 2006, Dell Inc
  All rights reserved.
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:
  - Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer.
    
  - Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution. 
  - Neither the name of Dell Inc nor the names of its contributors
    may be used to endorse or promote products derived from this software 
    without specific prior written permission. 
  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE. 
 
 
*****************************************************************************/
#include <ipmitool/cssipmi.h>
#include <ipmitool/cssapi.h>
#include <ipmitool/csssupt.h>
#include <ipmitool/cssstat.h>

/****************************************************************************
    Local Defines
****************************************************************************/
#define MAX_SENSOR_PER_SDR                      (16)
#define IPMI_FLAGS_SCANNING_DISABLED            BIT(6)
#define IPMI_FLAGS_INIT_UPDATE                  BIT(5)
#define MAX_PROBE_NAME                          (32)

/****************************************************************************
    Local Function Prototypes
****************************************************************************/

static long CSSCalcTenExponent(
    _IN long val, 
    _IN long exponent);

static long CSSConvertValues(
    _IN short           val, 
    _IN const void*     pSdr, 
    _IN unsigned char   convertType);
    
static void CSSConvertValueToStr(
    _OUT    char*           pStr,
    _IN     int             thrVal,
    _IN     unsigned char   sensorType);
    
static unsigned short ConvertToEventData(
    unsigned short  sensorState,
    unsigned short  readingMask,
    unsigned char   sensorType,
    unsigned char   readingType);
    
static char *CSSGetUnitsStr(_IN unsigned char sensorUnitType);


/****************************************************************************
    Global Data
****************************************************************************/
    
extern SensorStateElement g_SensorGenericTable[12];
extern SensorStateElement g_SensorSpecificTable[32];
extern char* g_SensorUnitsTable[];
    

/****************************************************************************
    Local Data
****************************************************************************/
static CSDDUSERAPI CSDDUSERAPIList;

/* C0 System Performace degradation status */
#define SENSOR_SPECIFICTYPEC0_SIZE  (6)
AppendStateText g_SensorSpecificTypeC0[SENSOR_SPECIFICTYPEC0_SIZE] =
{
    {CSS_SEVERITY_NORMAL,   CSS_SEVERITY_NORMAL,"restored"},
    {CSS_SEVERITY_NORMAL,   CSS_SEVERITY_NORMAL,"other"},
    {CSS_SEVERITY_NORMAL,   CSS_SEVERITY_NORMAL,"thermal protection"},
    {CSS_SEVERITY_NORMAL,   CSS_SEVERITY_NORMAL,"cooling capacity change"},
    {CSS_SEVERITY_NORMAL,   CSS_SEVERITY_NORMAL,"power capacity"},
    {CSS_SEVERITY_NORMAL,   CSS_SEVERITY_NORMAL,"user defined power capacity"}
};

/*  Code Begins */


/****************************************************************************
    CSDDAttach
****************************************************************************/
int CSDDAttach(CSDDUSERAPI *pfnApiList)
{
    CSDDUSERAPIList.GetFirstSDR         = pfnApiList->GetFirstSDR;
    CSDDUSERAPIList.GetNextSDR          = pfnApiList->GetNextSDR;
    CSDDUSERAPIList.GetSensorReading    = pfnApiList->GetSensorReading;
    CSDDUSERAPIList.GetSensorThresholds = pfnApiList->GetSensorThresholds;
    CSDDUSERAPIList.Oem2IPMISDR         = pfnApiList->Oem2IPMISDR;
    return COMMON_STATUS_SUCCESS;
}


/****************************************************************************
    CSDDDetach
****************************************************************************/
int CSDDDetach(void)
{
    return COMMON_STATUS_SUCCESS;
}


/****************************************************************************
    CSDDSensorList
****************************************************************************/
int CSDDGetSensorsTobeMonitored (
    _INOUT CSDDSensorList* pList,
    _IN const SDRType* pSdr, 
    _IN const EITable* pDonotMonitorEIList,
    _IN  void*         puserParameter)
{
    unsigned char   entityID;
    unsigned char   entityInst;
    unsigned char   shareCount;
    unsigned char   instance = 0;
    unsigned char   sensorNumber;
    unsigned char   sensorOwner;
    unsigned char   sensorFlags;
    unsigned char   recordType;
    int             eCount = 0;
    int             status = COMMON_STATUS_FAILURE;
    IPMISensorReadingType sensorReadingData;

    /* check for bad parameters */
    if ((NULL != pList) && (NULL != pSdr))
    {
        /* clear sensor count */
        pList->sensorCount = 0;
        status = COMMON_STATUS_SUCCESS;

        recordType = CSSSDRGetAttribute(
            pSdr, 
            SDR_ATTRIBUTE_RECORD_TYPE,
            CSDDUSERAPIList.Oem2IPMISDR);

        /* first check to see if this SDR is of type FULL or COMPACT */
        if ((SDR_COMPACT_SENSOR == recordType) || 
            (SDR_FULL_SENSOR ==  recordType))
        {
            /* check the entity and see if the entity 
            pointed to by this SDR needs to be 
            monitored or not */
            if (NULL != pDonotMonitorEIList)
            {
                entityID = CSSSDRGetAttribute(
                    pSdr,
                    SDR_ATTRIBUTE_ENTITY_ID,
                    CSDDUSERAPIList.Oem2IPMISDR);
                entityInst = CSSSDRGetAttribute(
                    pSdr,
                    SDR_ATTRIBUTE_ENTITY_INST,
                    CSDDUSERAPIList.Oem2IPMISDR);
                for (eCount = 0; eCount < pDonotMonitorEIList->listsize; eCount++)
                {
                    if ((entityID == pDonotMonitorEIList->eilist[eCount].entityID)
                        && (entityInst == pDonotMonitorEIList->eilist[eCount].entityInst))
                    {
                        return status;
                    }
                }
            }
            /* Get sensor sharing count From SDR */
            shareCount   = CSSSDRGetAttribute(
                pSdr,
                SDR_ATTRIBUTE_SHARE_COUNT,
                CSDDUSERAPIList.Oem2IPMISDR);

            sensorNumber = CSSSDRGetAttribute(
                pSdr,
                SDR_ATTRIBUTE_SENSOR_NUMBER,
                CSDDUSERAPIList.Oem2IPMISDR);

            sensorOwner  = CSSSDRGetAttribute(
                pSdr,
                SDR_ATTRIBUTE_SENSOR_OWNER_ID,
                CSDDUSERAPIList.Oem2IPMISDR);

            for(instance = 0; instance < shareCount; instance++)
            {
                /* query the sensor and see if the sensor is responding OK */
                status = CSDDUSERAPIList.GetSensorReading(sensorOwner,
                            (unsigned char )(sensorNumber + instance),
                            &sensorReadingData, puserParameter);
               
                if (status == COMMON_STATUS_SUCCESS) 
                {
                    /* Now we have the data, check the flags to see if this sensor 
                     needs to be ignored or the BMC has a problem */
                    sensorFlags = sensorReadingData.sensorFlags;
                
                    if (!(sensorFlags & IPMI_FLAGS_SCANNING_DISABLED)) 
                    {
                        /* we cannot add this sensor as it's disable to scan */
                        continue;
                    }

                    pList->sensorNumber[pList->sensorCount] = 
                        (unsigned char )(sensorNumber + instance);
                    pList->sensorCount++;
                }
                else if (status == COMMON_STATUS_FAILURE)
                {
                    continue;
                }
                /* Unexpected error?, break with error from GetSensorReading */
                else
                {
                    /* clear sensor count we have and error */
                    pList->sensorCount = 0;
                    break;
                }
            }
            /* check for good list */
            if (pList->sensorCount > 0)
            {
                status = COMMON_STATUS_SUCCESS;
            }
        }
    }    
    return status;  
}

/****************************************************************************
    CSDDGetSensorStaticInformation
****************************************************************************/
int CSDDGetSensorStaticInformation (
    _IN     const SDRType*  pSdr,
    _IN     unsigned char   sensorNamePolicy,
    _OUT    unsigned char*  pSensorReadingType, 
    _OUT    unsigned char*  pSensorType,
    _INOUT  short*          pSensorNameStrSize,
    _OUT    char*           pSensorNameStr,
    _INOUT  short*          pSensorTypeStrSize,
    _OUT    char*           pSensorTypeStr,
    _INOUT  short*          pUnitsStrSize,
    _OUT    char*           pUnitsStr,
    _IN     int             instance,
    _IN     void*           puserParameter)
{
    char*           tmpStr;
    short           tmpSize;
    SDRType*        pFruSdr = NULL;
    char            probeName[MAX_PROBE_NAME];
    char            entityName[MAX_PROBE_NAME];
    unsigned char   sensorNumber;

    UNREFERENCED_PARAMETER(sensorNamePolicy);
 
    if (NULL != pSensorReadingType)
    {
        *pSensorReadingType = CSSSDRGetAttribute(
            pSdr,
            SDR_ATTRIBUTE_READING_TYPE,
            CSDDUSERAPIList.Oem2IPMISDR);
    }

    if (NULL != pSensorType)
    {
        *pSensorType = CSSSDRGetAttribute(
            pSdr,
            SDR_ATTRIBUTE_SENSOR_TYPE,
            CSDDUSERAPIList.Oem2IPMISDR);
    }
    
    if ( (NULL != pSensorTypeStr) && 
         (NULL != pSensorTypeStrSize) )
    {
        tmpStr  = CSSGetSensorTypeStr(
            CSSSDRGetAttribute(
            pSdr,
            SDR_ATTRIBUTE_SENSOR_TYPE,
            CSDDUSERAPIList.Oem2IPMISDR),
            CSSSDRGetAttribute(
            pSdr,
            SDR_ATTRIBUTE_READING_TYPE,
            CSDDUSERAPIList.Oem2IPMISDR));
                        
        tmpSize = (short)(strlen(tmpStr) + 1);
        
        if (tmpSize <= *pSensorTypeStrSize)
        {
            memcpy(pSensorTypeStr, tmpStr, tmpSize);
        }
        *pSensorTypeStrSize = tmpSize;
    }
    
    if ( (NULL != pSensorNameStr) && 
         (NULL != pSensorNameStrSize))
    {
        pFruSdr = CSSFindEntitySDRRecord(
            CSDDUSERAPIList.GetFirstSDR,
            CSDDUSERAPIList.GetNextSDR,
            CSDDUSERAPIList.Oem2IPMISDR,
            pSdr, 
            puserParameter);
        probeName[0]  = 0;
        entityName[0] = 0;
        sensorNumber = CSSSDRGetAttribute(
                pSdr, 
                SDR_ATTRIBUTE_SENSOR_NUMBER,
                CSDDUSERAPIList.Oem2IPMISDR);

        if ((unsigned char)instance > sensorNumber)
        {
            sensorNumber = (unsigned char)(instance - sensorNumber);
        }
        else
        {
            sensorNumber = 0;
        }

        CSSGetProbeName(
            (void*)pSdr, 
            (unsigned char)sensorNumber, 
            probeName, 
            MAX_PROBE_NAME,
            CSDDUSERAPIList.Oem2IPMISDR);
        
        if (NULL != pFruSdr)
        {
            CSSGetProbeName(
                (void*)pFruSdr, 
                0, 
                entityName, 
                MAX_PROBE_NAME,
                CSDDUSERAPIList.Oem2IPMISDR);
        }
        
        tmpSize = (short)(strlen(probeName) + strlen(entityName) + 1);
        
        if (tmpSize <= *pSensorNameStrSize)
        {
            if (entityName[0] != 0)
            {
                strcpy(pSensorNameStr, entityName);
                strcat(pSensorNameStr, " ");
                strcat(pSensorNameStr, probeName);
            }
            else
            {
                strcpy(pSensorNameStr, probeName);
            }
            
        }
        *pSensorNameStrSize = tmpSize;
    }

    if ( (NULL != pUnitsStr) && 
         (NULL !=  pUnitsStrSize) )
    {
        tmpStr = 
            CSSGetUnitsStr(
                CSSSDRGetAttribute(
                    pSdr,
                    SDR_ATTRIBUTE_SENSOR_UNITS,
                    CSDDUSERAPIList.Oem2IPMISDR));
        
        tmpSize = (short)(strlen(tmpStr) + 1);
        
        if (tmpSize <= *pUnitsStrSize)
        {
            strcpy(pUnitsStr, tmpStr);
        }
        *pUnitsStrSize = tmpSize;
    }
       
    return COMMON_STATUS_SUCCESS;
}

    
/****************************************************************************
    CSDDGetSensorDynamicInformation
****************************************************************************/
int CSDDGetSensorDynamicInformation (
    _IN     const SDRType*  pSdr,
    _OUT    long*           pSensorReading,
    _OUT    unsigned short* pSensorState,
    _INOUT  short*          pSensorReadingStrSize,
    _OUT    char*           pSensorReadingStr,
    _INOUT  short*          pSensorStateStrSize,
    _OUT    char*           pSensorStateStr,
    _OUT    short*          pSeverity,
    _IN     int             instance,
    _IN     void*           puserParameter)
{
    IPMISDR*                pIPMISdr;
    unsigned char           sensorOwner;
    unsigned char           sensorNumber;
    unsigned char           sensorType;
    unsigned char           readingType;
    unsigned short          eventData;
    int                     status;
    IPMISensorReadingType   sensorReadingData;
    long                    sensorReading;
    int                     strLen = 0;
    unsigned char           total;
    unsigned char           index;
    unsigned short          readingMask;
    char                    sensorReadingStr[16];

    UNREFERENCED_PARAMETER( pSensorStateStrSize );

    sensorOwner  = CSSSDRGetAttribute(
        pSdr,
        SDR_ATTRIBUTE_SENSOR_OWNER_ID,
        CSDDUSERAPIList.Oem2IPMISDR);
    sensorNumber = CSSSDRGetAttribute(
        pSdr,
        SDR_ATTRIBUTE_SENSOR_NUMBER,
        CSDDUSERAPIList.Oem2IPMISDR);
    readingType  = CSSSDRGetAttribute(
        pSdr,
        SDR_ATTRIBUTE_READING_TYPE,
        CSDDUSERAPIList.Oem2IPMISDR);
    sensorType   = CSSSDRGetAttribute(
        pSdr,
        SDR_ATTRIBUTE_SENSOR_TYPE,
        CSDDUSERAPIList.Oem2IPMISDR);
    
    pIPMISdr = (IPMISDR*)pSdr;
    readingMask = pIPMISdr->type.type2.readingMask & 0x7FFF;
    if (pIPMISdr->header.recordType == SDR_FULL_SENSOR)
    {
        readingMask = pIPMISdr->type.type1.readingMask & 0x7FFF;
    }

    if ((unsigned char)instance > sensorNumber)
    {
        sensorNumber = (unsigned char)(instance);
    }
    /* query the sensor it's current state */
    status = CSDDUSERAPIList.GetSensorReading(
        sensorOwner,
        sensorNumber,
        &sensorReadingData, puserParameter);
        
    if (status != COMMON_STATUS_SUCCESS) 
    {
        return status;
    }

    if (sensorReadingData.sensorFlags & IPMI_FLAGS_INIT_UPDATE)
    {
        return COMMON_STATUS_DEVICE_NOT_READY;
    }

    /* set sensor state */
    sensorReadingData.sensorState &= readingMask;
    if (NULL != pSensorState)
    {        
        *pSensorState = sensorReadingData.sensorState;
    }
    
    if ( readingType == IPMI_READING_TYPE_THRESHOLD )
    {
        sensorReading = CSSConvertValues(
            sensorReadingData.sensorReading, 
            pSdr, 
            TO_COOKED);

        if (NULL != pSensorReading)
        {
            *pSensorReading = sensorReading;
        }
        
        if ( (NULL != pSensorReadingStr) &&
             (NULL != pSensorReadingStrSize) )
        {
            CSSConvertValueToStr(
                sensorReadingStr, 
                (short)sensorReading, 
                sensorType);
            
            strLen = strlen(sensorReadingStr) + 1;
            if (strLen <= *pSensorReadingStrSize)
            {
                strcpy(pSensorReadingStr, sensorReadingStr);
            }
            *pSensorReadingStrSize = (short)strLen;
        }
        /* Get severity */
        if (NULL != pSeverity)
        {
            *pSeverity = CSS_SEVERITY_NORMAL;
            /* Check most sever first, IPMI spec allows multible bits to be set */
            /* Upper and lower Non-Recoverable? */
            if (sensorReadingData.sensorState & 
                (IPMI_UNR_THRESHOLD_MASK|IPMI_LNR_THRESHOLD_MASK))
            {
                *pSeverity = CSS_SEVERITY_CRITICAL;
            } 
            /* Upper and lower Critical? */
            else if (sensorReadingData.sensorState & 
                (IPMI_UC_THRESHOLD_MASK|IPMI_LC_THRESHOLD_MASK))
            {
                *pSeverity = CSS_SEVERITY_CRITICAL;
            }
            /* Upper and lower Non-Critical? */
            else if (sensorReadingData.sensorState & 
                (IPMI_UNC_THRESHOLD_MASK|IPMI_LNC_THRESHOLD_MASK))
            {
                *pSeverity = CSS_SEVERITY_WARNING;
            }            
        }
    }
    else
    {
        eventData = ConvertToEventData(
            sensorReadingData.sensorState,
            readingMask,
            sensorType,
            readingType);
                
        /* Generic Sensor reading type */
        if ( (readingType > (unsigned char)0) && 
             (readingType < (unsigned char)MAX_READING_TYPES) )
        {
            total = sizeof(g_SensorGenericTable)/sizeof(g_SensorGenericTable[0]);
            for (index = 0; index < total; index++)
            {
                /* Match the reading type */
                if (g_SensorGenericTable[index].readingType == readingType)
                {
                    /* correct index? */
                    if (eventData < g_SensorGenericTable[index].maxIndex)
                    {                
                        strcpy(
                            pSensorStateStr, 
                            g_SensorGenericTable[index].pSensorState[eventData].state);
                        *pSeverity = 
                            g_SensorGenericTable[index].pSensorState[eventData].severity;
                        break;
                    }
                }
            }
        }
        /* Sensor Specific reading type */
        else if (readingType == IPMI_READING_TYPE_SPEC_OFFSET)
        {
			if (IPMI_SENSOR_SYS_DEGRADE_STATUS == sensorType)
			{
				// special case for performance sensor
				if (eventData < SENSOR_SPECIFICTYPEC0_SIZE)
				{
					strcpy (pSensorStateStr, g_SensorSpecificTypeC0[eventData].state);
					*pSeverity = g_SensorSpecificTypeC0[eventData].assertSeverity;
				}
			}
			else
			{
				total = sizeof(g_SensorSpecificTable)/sizeof(g_SensorSpecificTable[0]);
				for (index = 0; index < total; index++)
				{
					/* Match the reading type */
					if (g_SensorSpecificTable[index].readingType == sensorType)
					{
						/* correct index? */
						if (eventData < g_SensorSpecificTable[index].maxIndex)
						{ 
							strcpy(
								pSensorStateStr, 
								g_SensorSpecificTable[index].pSensorState[eventData].state);
							*pSeverity = 
								g_SensorSpecificTable[index].pSensorState[eventData].severity;
							break;
						}
					}
				}
			}
        }                
        /* OEM */
        else if ((readingType > MIN_OEM_READING_TYPES) && 
            (readingType <= (unsigned char)MAX_OEM_READING_TYPES))
        {
            if (readingType == OEM_DELL_READING_TYPE)
            {
                *pSeverity = CSS_SEVERITY_NORMAL;
                /* OEM error register pointer */
                strcpy(pSensorStateStr,"OEM Diagnostic data event");
            }
        }
    }
            
    return COMMON_STATUS_SUCCESS;
}


/****************************************************************************
    CSDDGetSensorThresholds 
****************************************************************************/        
int CSDDGetSensorThresholds (
    _IN     const SDRType*      pSdr,
    _OUT    unsigned char*      pThresholdMask,
    _OUT    SensorThrInfo*      pThrData,
    _OUT    SensorThrStrType*   pThrStrData,
    _IN     int                 locale,
    _IN     void*               puserParameter)
{
    IPMISensorThresholdType thrData;
    unsigned int probeCaps = 0;
    unsigned char thrReadMask;
    unsigned char thrSetMask;
    unsigned char sensorType;

    UNREFERENCED_PARAMETER( locale );

    if ( COMMON_STATUS_SUCCESS == CSDDUSERAPIList.GetSensorThresholds(
        CSSSDRGetAttribute(pSdr,SDR_ATTRIBUTE_SENSOR_OWNER_ID,CSDDUSERAPIList.Oem2IPMISDR),
        CSSSDRGetAttribute(pSdr,SDR_ATTRIBUTE_SENSOR_NUMBER,CSDDUSERAPIList.Oem2IPMISDR),
        &thrData, puserParameter))
    {
        sensorType = CSSSDRGetAttribute(
            pSdr,
            SDR_ATTRIBUTE_SENSOR_TYPE,
            CSDDUSERAPIList.Oem2IPMISDR);

        /* thrReadMask can be used to find out what thresholds are readable */
        thrReadMask = CSSSDRGetAttribute(
            pSdr,
            SDR_ATTRIBUTE_THRES_READ_MASK,
            CSDDUSERAPIList.Oem2IPMISDR);

        /* thrSetMask can be used to find out what thresholds are settable */
        thrSetMask = CSSSDRGetAttribute(
            pSdr,
            SDR_ATTRIBUTE_THRES_SET_MASK,
            CSDDUSERAPIList.Oem2IPMISDR);
        
        /* is the upper critical threshold valid ? */
        if ((thrReadMask & IPMI_UC_THRESHOLD_MASK) != 0)
        {
            pThrData->ucThr = CSSConvertValues(thrData.ucThr, 
                                            pSdr, TO_COOKED);
            CSSConvertValueToStr(
                pThrStrData->ucThr,
                (short)pThrData->ucThr,sensorType);
        } 
        else
        {
            strcpy(pThrStrData->ucThr, "N/A");
            probeCaps |= SENSOR_THR_UC_DISABLED; 
        }
        
        /* is the lower critical threshold valid ? */
        if ((thrReadMask & IPMI_LC_THRESHOLD_MASK) != 0)
        {
            pThrData->lcThr = CSSConvertValues(thrData.lcThr, 
                                                    pSdr, TO_COOKED);
            CSSConvertValueToStr(
                pThrStrData->lcThr,
                (short)pThrData->lcThr,sensorType);
        }
        else
        {
            strcpy(pThrStrData->lcThr, "N/A");
            probeCaps |= SENSOR_THR_LC_DISABLED;
        }
        
        /*is the upper non critical threshold valid ?*/
        if ((thrReadMask & IPMI_UNC_THRESHOLD_MASK) != 0)
        {
            /* is upper non critical threshold settable */
            if ((thrSetMask & IPMI_UNC_THRESHOLD_MASK) != 0)
            {
                probeCaps |= SENSOR_THR_UNC_SETS_ENABLE | 
                                SENSOR_THR_UNC_SETS_ENABLE;
            }
            
            pThrData->uncThr = CSSConvertValues(thrData.uncThr, 
                                                    pSdr, TO_COOKED);
            CSSConvertValueToStr(
                pThrStrData->uncThr,
                (short)pThrData->uncThr,sensorType);
        }
        else
        {
            strcpy(pThrStrData->uncThr, "N/A");
            probeCaps |= SENSOR_THR_UNC_DISABLED;
        }
        
        /* is the lower non critical threshold valid ?*/
        if ((thrReadMask & IPMI_LNC_THRESHOLD_MASK) != 0)
        {
            if ((thrSetMask & IPMI_LNC_THRESHOLD_MASK) != 0)
            {
                probeCaps |= SENSOR_THR_LNC_SETS_ENABLE | 
                                SENSOR_THR_LNC_SETS_ENABLE;
            } 
            
            pThrData->lncThr = CSSConvertValues(thrData.lncThr, 
                                                    pSdr, TO_COOKED);
            CSSConvertValueToStr(
                pThrStrData->lncThr,
                (short)pThrData->lncThr,sensorType);
        }
        else
        {
            strcpy(pThrStrData->lncThr, "N/A");
            probeCaps |= SENSOR_THR_LNC_DISABLED;
        }
        if (NULL != pThresholdMask)
        {
            *pThresholdMask = (unsigned char)probeCaps;
        }
    }
    return COMMON_STATUS_SUCCESS;
}

/****************************************************************************
    CalcTenExponent
    This routine takes a value and multiplies it by 10 to the exponent 
    value
****************************************************************************/
static long CSSCalcTenExponent(long val, long exponent) 
{
    if (exponent > 0) 
    {
        while (exponent > 0) 
        {
            val *= 10;
            exponent--;
        }
    }
    else if (exponent < 0) 
    {
        while (exponent < 0) 
        {
            val /= 10;
            exponent++;
        }
    }
    return (val);
}

/****************************************************************************
    CSSConvertValues
    This routine converts raw values to cooked or cooked to raw
****************************************************************************/
static long CSSConvertValues(
    _IN short           val, 
    _IN const void*     pSdr, 
    _IN unsigned char   convertType)
{
    short           M;
    short           B;
    char            K1;
    char            K2;
    long            convertValue    = 0;
    unsigned char   unitMod         = 0;

    /* get the exponent that is used to convert the value 
     from its native units to the units
     used by DMI.  (volts->millvolts, etc) */
    switch (CSSSDRGetAttribute(
                pSdr,
                SDR_ATTRIBUTE_SENSOR_TYPE,
                CSDDUSERAPIList.Oem2IPMISDR)) 
    {
        case IPMI_SENSOR_TEMP:
            unitMod = 1;
        break;

        case IPMI_SENSOR_VOLT:
            unitMod = 3;
        break;

		case IPMI_SENSOR_CURRENT:
            /* check if sensor uints are Watts */
            if (CSSSDRGetAttribute(
                pSdr,
                SDR_ATTRIBUTE_SENSOR_UNITS,
                CSDDUSERAPIList.Oem2IPMISDR) == 6)
            {
              unitMod = 0;
            }
            else
            {
              unitMod = 1;
            }
        break;
    }

    /* See the ipmi spec for the formula to 
     convert raw readings */
    M = (short) ((((unsigned short)(CSSSDRGetAttribute(pSdr,SDR_ATTRIBUTE_TOLERANCE,CSDDUSERAPIList.Oem2IPMISDR) & 0xC0)) << 2) +
         CSSSDRGetAttribute(pSdr,SDR_ATTRIBUTE_MULTIPLIER,CSDDUSERAPIList.Oem2IPMISDR));

    /* 10 bit signed number.  If negative, then extend the sign bit */
    if (M & 0x0200)
    {
        M |= 0xFC00;
    }

    B = (short) ((((unsigned short)(CSSSDRGetAttribute(pSdr,SDR_ATTRIBUTE_ACCURACY,CSDDUSERAPIList.Oem2IPMISDR) & 0xC0)) << 2) + 
        CSSSDRGetAttribute(pSdr,SDR_ATTRIBUTE_OFFSET,CSDDUSERAPIList.Oem2IPMISDR));

    /* 10 bit signed number.  If negative, then extend the sign bit */
    if (B & 0x0200)
    {
        B |= 0xFC00;
    }

    K1 = (char) ((CSSSDRGetAttribute(pSdr,SDR_ATTRIBUTE_EXPONENT,CSDDUSERAPIList.Oem2IPMISDR) & 0x0F));
    if (K1 &0x08)
    {
        K1 |= 0xf0;
    }

    K2 = (char) (CSSSDRGetAttribute(pSdr,SDR_ATTRIBUTE_EXPONENT,CSDDUSERAPIList.Oem2IPMISDR) >> 4);
    if (K2 &0x08)
    {
        K2 |= 0xf0;
    }

    if (convertType == TO_COOKED) 
    {
        convertValue = CSSCalcTenExponent((long)(M * val), (long)(K2 + unitMod)) + 
            CSSCalcTenExponent((long)B, (long)(K1 + K2 + unitMod));
    }
    else 
    {
        convertValue = (CSSCalcTenExponent((long)val,(long)(-K2-unitMod)) - CSSCalcTenExponent((long)B,(long)K1))/M;
    }
    return convertValue;
}

/****************************************************************************
    CSSConvertValueToStr

 ****************************************************************************/
static void CSSConvertValueToStr(
    _OUT    char*           pStr,
    _IN     int             thrVal,
    _IN     unsigned char   sensorType)
{
    longdiv_t   value;
    char        wholeNum[16];
    char        remainder[16];

    switch (sensorType)
    {
        case IPMI_SENSOR_TEMP:
            value = CSSLongDiv((long)thrVal, 10);
            CSSlongToAscii(value.quot, wholeNum, 10);
            if (value.rem < 0)
            {
                value.rem = -value.rem;
            }            
            CSSlongToAscii(value.rem, remainder, 10);
            strcpy(pStr, wholeNum);
            strcat(pStr, ".");
            strcat(pStr, remainder);
        break;

        case IPMI_SENSOR_VOLT:
            value = CSSLongDiv((long)thrVal, 1000);
            CSSlongToAscii(value.quot, wholeNum, 10);
            if (value.rem < 0)
            {
                value.rem = -value.rem;
            }
            CSSlongToAscii(value.rem, remainder, 10);
            strcpy(pStr, wholeNum);
            strcat(pStr, ".");

            if (value.rem < 100)
            {
                strcat(pStr, "0");
            }
            if (value.rem < 10)
            {
                strcat(pStr, "0");
            }            
            strcat(pStr, remainder);
        break;

        default:
            CSSlongToAscii((long)thrVal, wholeNum, 10);
            strcpy(pStr, wholeNum);
        break;
    }
}

/****************************************************************************
    ConvertToEventData
****************************************************************************/
static unsigned short ConvertToEventData(
    unsigned short  sensorState,
    unsigned short  readingMask,
    unsigned char   sensorType,
    unsigned char   readingType)
{
    unsigned short  bit       = 0;
    unsigned short  mask      = 0x7FFF;
    unsigned short  eventData = 0;
        
    if (readingType == IPMI_READING_TYPE_SPEC_OFFSET)
    {
        /* Special cases */
        switch(sensorType)
        {
            case IPMI_SENSOR_BATTERY:
                if (sensorState == 0)
                {
                  /* check for present bit supported */
                   if ((readingMask & BIT(2)) == 0)
                   {
                       sensorState = BIT(2);
                   }
                }
            break;

            case IPMI_SENSOR_PS:
                /* present bit set? */
                if (sensorState & BIT(0))
                {
                    if (sensorState & 0x7FFE)
                    {
                        sensorState &= 0x7FFE;
                        if (sensorState & BIT(3))
                        {
                            sensorState = BIT(3);
                        }
                    }
                }
                else /* No PS */
                {
                    sensorState = 0;
                }
            break;

            case IPMI_SENSOR_DRIVE_SLOT:
            case IPMI_SENSOR_VRM:   /* NOTE: this is the RAC sensor */
                /* present bit set? */
                if (sensorState & BIT(0))
                {
                    /* other bits set? */
                    if (sensorState & 0x7FFE)
                    {
                        sensorState &= 0x7FFE;
                    }
                }
                else
                {
                    sensorState = 0;
                }
            break;
                
            case IPMI_SENSOR_PROCESSOR:
                /* present bit set? */
                if (sensorState & BIT(7))
                {
                    /* other bits set? */
                    if (sensorState & 0x7F7F)
                    {
                        sensorState &= 0x7F7F;
                    }                   
                }
                else
                {
                    sensorState = 0;
                }
            break;
            
            case IPMI_SENSOR_MEMORY:
                /* present bit set? */
                if (sensorState & BIT(6))
                {
                    /* other bits set? */
                    if (sensorState & 0x7FBF)
                    {
                        sensorState &= 0x7FBF;
                    }
                }
                else
                {
                    sensorState = 0;
                }
            break;
        }
    }
    if (sensorState & mask)
    {        
        eventData = 1;
        bit       = 0;
        do
        {
            if (sensorState & BIT(bit))
            {
                break;
            }
            bit++;
            eventData++;
        }while (eventData < (MAX_READING_BIT - 1));
    }
    return eventData;
}

/****************************************************************************
    CSSGetUnitsStr
   This function generates a string that describes the IPMI defined units
****************************************************************************/
char *CSSGetUnitsStr(_IN unsigned char sensorUnitType)
{
    char *unitsStr = NULL;
    
    if (sensorUnitType < MAX_UNITS_TYPE_NAMES)
    {
        unitsStr = g_SensorUnitsTable[sensorUnitType];
    }

    return unitsStr;
}



/* Code ends */


